@@ -68,8 +68,9 @@ find_jobs(int, cron_db *, int, int, long),
 set_time(int),
 cron_sleep(int, cron_db *),
 sigchld_handler(int),
-sighup_handler(int),
-sigchld_reaper(void), sigintterm_handler(int), parse_args(int c, char *v[]);
+sighup_handler(int ATTRIBUTE_UNUSED),
+sigchld_reaper(void),
+sigintterm_handler(int ATTRIBUTE_UNUSED), parse_args(int c, char *v[]);
 
 static volatile sig_atomic_t got_sighup, got_sigchld, got_sigintterm;
 static int timeRunning, virtualTime, clockTime;
@@ -188,6 +189,7 @@ static void usage(void) {
 	fprintf(stderr, " -P         use PATH=\"%s\"\n", _PATH_DEFPATH);
 	fprintf(stderr, " -c         enable clustering support\n");
 	fprintf(stderr, " -s         log into syslog instead of sending mails\n");
+	fprintf(stderr, " -V         print version and exit\n");
 	fprintf(stderr, " -x <flag>  print debug information\n");
 	fprintf(stderr, "\n");
 	fprintf(stderr, "Debugging flags are: ");
@@ -260,7 +262,7 @@ int main(int argc, char *argv[]) {
 	 */
 	setlocale(LC_ALL, "");	/* set locale to system defaults or to
 							 * that specified by any  LC_* env vars */
-	if ((cs = nl_langinfo(CODESET)) != 0L)
+	if ((cs = nl_langinfo(CODESET)) != NULL)
 		strncpy(cron_default_mail_charset, cs, MAX_ENVSTR-1);
 	else
 		strcpy(cron_default_mail_charset, "US-ASCII");
@@ -288,7 +290,6 @@ int main(int argc, char *argv[]) {
 				if (fd != STDERR)
 					(void) close(fd);
 			}
-			log_it("CRON", getpid(), "STARTUP", PACKAGE_VERSION, 0);
 			break;
 		default:
 			/* parent process should just die */
@@ -296,6 +297,8 @@ int main(int argc, char *argv[]) {
 		}
 	}
 
+	log_it("CRON", getpid(), "STARTUP", PACKAGE_VERSION, 0);
+
 	if (!SyslogOutput && MailCmd[0] == '\0' && access("/usr/sbin/sendmail", X_OK) != 0) {
 		SyslogOutput=1;
 		log_it("CRON", pid, "INFO","Syslog will be used instead of sendmail.", 0);
@@ -306,8 +309,8 @@ int main(int argc, char *argv[]) {
 	/* obtain a random scaling factor for RANDOM_DELAY */
 	if (gettimeofday(&tv, &tz) != 0)
 		tv.tv_usec = 0;
-	srandom(pid + tv.tv_usec);
-	RandomScale = random() / (double)RAND_MAX;
+	srandom((unsigned int)(pid + tv.tv_usec));
+	RandomScale = (double)random() / (double)RAND_MAX;
 	snprintf(buf, sizeof(buf), "RANDOM_DELAY will be scaled with factor %d%% if used.", (int)(RandomScale*100));
 	log_it("CRON", pid, "INFO", buf, 0);
 
@@ -555,15 +558,15 @@ static void find_jobs(int vtime, cron_db * db, int doWild, int doNonWild, long v
 
 	orig_tz = getenv("TZ");
 
-		/* the dom/dow situation is odd.  '* * 1,15 * Sun' will run on the
-		 * first and fifteenth AND every Sunday;  '* * * * Sun' will run *only*
-		 * on Sundays;  '* * 1,15 * *' will run *only* the 1st and 15th.  this
-		 * is why we keep 'e->dow_star' and 'e->dom_star'.  yes, it's bizarre.
-		 * like many bizarre things, it's the standard.
-		 */
-		for (u = db->head; u != NULL; u = u->next) {
+	/* the dom/dow situation is odd.  '* * 1,15 * Sun' will run on the
+	 * first and fifteenth AND every Sunday;  '* * * * Sun' will run *only*
+	 * on Sundays;  '* * 1,15 * *' will run *only* the 1st and 15th.  this
+	 * is why we keep 'e->dow_star' and 'e->dom_star'.  yes, it's bizarre.
+	 * like many bizarre things, it's the standard.
+	 */
+	for (u = db->head; u != NULL; u = u->next) {
 		for (e = u->crontab; e != NULL; e = e->next) {
-			time_t virtualSecond = (vtime - e->delay) * SECONDS_PER_MINUTE;
+			time_t virtualSecond = (time_t)(vtime - e->delay) * (time_t)SECONDS_PER_MINUTE;
 			time_t virtualGMTSecond = virtualSecond - vGMToff;
 			job_tz = env_get("CRON_TZ", e->envp);
 			maketime(job_tz, orig_tz);
@@ -614,7 +617,7 @@ static void set_time(int initialize) {
 		GMToff = get_gmtoff(&StartTime, &tm);
 		Debug(DSCH, ("[%ld] GMToff=%ld\n", (long) getpid(), (long) GMToff));
 	}
-	clockTime = (StartTime + GMToff) / (time_t) SECONDS_PER_MINUTE;
+	clockTime = (int)((StartTime + GMToff) / (time_t) SECONDS_PER_MINUTE);
 }
 
 /*
@@ -649,15 +652,15 @@ static void cron_sleep(int target, cron_db * db) {
 	}
 }
 
-static void sighup_handler(int x) {
+static void sighup_handler(int x ATTRIBUTE_UNUSED) {
 	got_sighup = 1;
 }
 
-static void sigchld_handler(int x) {
+static void sigchld_handler(int x ATTRIBUTE_UNUSED) {
 	got_sigchld = 1;
 }
 
-static void sigintterm_handler(int x) {
+static void sigintterm_handler(int x ATTRIBUTE_UNUSED) {
 	got_sigintterm = 1;
 }
 
@@ -672,15 +675,15 @@ static void sigchld_reaper(void) {
 			if (errno == EINTR)
 				continue;
 			Debug(DPROC, ("[%ld] sigchld...no children\n", (long) getpid()));
-				break;
+			break;
 		case 0:
 			Debug(DPROC, ("[%ld] sigchld...no dead kids\n", (long) getpid()));
-				break;
+			break;
 		default:
 			Debug(DPROC,
 				("[%ld] sigchld...pid #%ld died, stat=%d\n",
 					(long) getpid(), (long) pid, WEXITSTATUS(waiter)));
-				break;
+			break;
 		}
 	} while (pid > 0);
 }
@@ -688,7 +691,7 @@ static void sigchld_reaper(void) {
 static void parse_args(int argc, char *argv[]) {
 	int argch;
 
-	while (-1 != (argch = getopt(argc, argv, "hnpsiPx:m:c"))) {
+	while (-1 != (argch = getopt(argc, argv, "hnpsiPx:m:cV"))) {
 		switch (argch) {
 			case 'x':
 				if (!set_debug_flags(optarg))
@@ -715,6 +718,9 @@ static void parse_args(int argc, char *argv[]) {
 			case 'c':
 				EnableClustering = 1;
 				break;
+			case 'V':
+				puts(PACKAGE_STRING);
+				exit(EXIT_SUCCESS);
 			case 'h':
 			default:
 				usage();
