@@ -66,21 +66,24 @@
 
 #define NHEADER_LINES 0
 
+#define COMMENT_COLOR  "\x1B[34m"
+#define RESET_COLOR "\033[0m"
+
 enum opt_t {opt_unknown, opt_list, opt_delete, opt_edit, opt_replace, opt_hostset, opt_hostget};
 
 #if DEBUGGING
 static const char *Options[] = {"???", "list", "delete", "edit", "replace", "hostset", "hostget"};
 
 # ifdef WITH_SELINUX
-static const char *getoptargs = "u:lerisncx:";
+static const char *getoptargs = "u:lerisncx:V";
 # else
-static const char *getoptargs = "u:lerincx:";
+static const char *getoptargs = "u:lerincx:V";
 # endif
 #else
 # ifdef WITH_SELINUX
-static const char *getoptargs = "u:lerisnc";
+static const char *getoptargs = "u:lerisncV";
 # else
-static const char *getoptargs = "u:lerinc";
+static const char *getoptargs = "u:lerincV";
 # endif
 #endif
 #ifdef WITH_SELINUX
@@ -125,6 +128,7 @@ static void usage(const char *msg) {
 #ifdef WITH_SELINUX
 	fprintf(stderr, " -s         selinux context\n");
 #endif
+	fprintf(stderr, " -V         print version and exit\n");
 #ifdef DEBUGGING
 	fprintf(stderr, " -x <mask>  enable debugging\n");
 #endif
@@ -134,8 +138,6 @@ static void usage(const char *msg) {
 
 int main(int argc, char *argv[]) {
 	int exitstatus;
-	char n[] = "-";	/*set the n string to - so we have a valid string to use */
-	char *nargv[] = { argv[0], n, NULL };
 
 	if ((ProgramName=strrchr(argv[0], '/')) == NULL) {
 		ProgramName = argv[0];
@@ -153,10 +155,6 @@ int main(int argc, char *argv[]) {
 #if defined(BSD)
 	setlinebuf(stderr);
 #endif
-	/*should we desire to make changes to behavior later. */
-	if (argv[1] == NULL) {	/* change behavior to allow crontab to take stdin with no '-' */
-		argv = nargv;
-	}
 	parse_args(argc, argv);	/* sets many globals, opens a file */
 	check_spool_dir();
 	if (!allowed(RealUser, CRON_ALLOW, CRON_DENY)) {
@@ -317,6 +315,9 @@ static void parse_args(int argc, char *argv[]) {
 			}
 			Option = opt_hostget;
 			break;
+		case 'V':
+			puts(PACKAGE_STRING);
+			exit(EXIT_SUCCESS);
 		default:
 			usage("unrecognized option");
 		}
@@ -324,7 +325,7 @@ static void parse_args(int argc, char *argv[]) {
 
 	endpwent();
 
-	if (Option == opt_hostset && argv[optind] != NULL) {            
+	if (Option == opt_hostset && argv[optind] != NULL) {
 		HostSpecified = 1;
 		if (strlen(argv[optind]) >= sizeof Host)
 			usage("hostname too long");
@@ -344,7 +345,7 @@ static void parse_args(int argc, char *argv[]) {
 			(void) strcpy(Filename, argv[optind]);
 		}
 		else
-			usage("file name must be specified for replace");
+			usage("file name or - (for stdin) must be specified for replace");
 	}
 
 	if (Option == opt_replace) {
@@ -358,6 +359,7 @@ static void parse_args(int argc, char *argv[]) {
 			 * Arnt Gulbrandsen <agulbra@pvv.unit.no> for spotting
 			 * the race.
 			 */
+			struct stat sb;
 
 			if (swap_uids() < OK) {
 				perror("swapping uids");
@@ -367,6 +369,17 @@ static void parse_args(int argc, char *argv[]) {
 				perror(Filename);
 				exit(ERROR_EXIT);
 			}
+			if (fstat(fileno(NewCrontab), &sb) < 0) {
+				perror(Filename);
+				exit(ERROR_EXIT);
+			}
+			if ((sb.st_mode & S_IFMT) == S_IFDIR) {
+				fprintf(stderr,
+					"cannot replace crontab with a directory: %s\n",
+					Filename);
+				fclose(NewCrontab);
+				exit(ERROR_EXIT);
+			}
 			if (swap_uids_back() < OK) {
 				perror("swapping uids back");
 				exit(ERROR_EXIT);
@@ -382,6 +395,8 @@ static void list_cmd(void) {
 	char n[MAX_FNAME];
 	FILE *f;
 	int ch;
+	const int is_tty = isatty(STDOUT);
+	int new_line = 1;
 
 	log_it(RealUser, Pid, "LIST", User, 0);
 	if (!glue_strings(n, sizeof n, SPOOL_DIR, User, '/')) {
@@ -399,8 +414,18 @@ static void list_cmd(void) {
 	/* file is open. copy to stdout, close.
 	 */
 	Set_LineNum(1)
-		while (EOF != (ch = get_char(f)))
+	while (EOF != (ch = get_char(f))) {
+		if (is_tty && new_line) {
+			if (ch == '#') {
+				fputs(COMMENT_COLOR, stdout);
+			}
+			else {
+				fputs(RESET_COLOR, stdout);
+			}
+		}
 		putchar(ch);
+		new_line = ch == '\n';
+	}
 	fclose(f);
 }
 
@@ -409,7 +434,7 @@ static void delete_cmd(void) {
 	if (PromptOnDelete == 1) {
 		printf("crontab: really delete %s's crontab? ", User);
 		fflush(stdout);
-		if ((fgets(n, MAX_FNAME - 1, stdin) == 0L)
+		if ((fgets(n, MAX_FNAME - 1, stdin) == NULL)
 			|| ((n[0] != 'Y') && (n[0] != 'y'))
 			)
 			exit(0);
@@ -564,10 +589,10 @@ static void edit_cmd(void) {
 		perror(Filename);
 		exit(ERROR_EXIT);
 	}
-        if (swap_uids() == -1) {
-                perror("swapping uids");
-                exit(ERROR_EXIT);
-        }
+	if (swap_uids() == -1) {
+		perror("swapping uids");
+		exit(ERROR_EXIT);
+	}
 	/* Set it to 1970 */
 	utimebuf.actime = 0;
 	utimebuf.modtime = 0;
@@ -690,7 +715,7 @@ static void edit_cmd(void) {
 		perror("swapping uids back");
 		exit(ERROR_EXIT);
 	}
-	if (NewCrontab == 0L) {
+	if (NewCrontab == NULL) {
 		perror("fopen");
 		goto fatal;
 	}
@@ -699,10 +724,10 @@ static void edit_cmd(void) {
 		break;
 	case -1:
 		for (;;) {
-			printf("Do you want to retry the same edit? ");
+			printf("Do you want to retry the same edit? (Y/N) ");
 			fflush(stdout);
 			q[0] = '\0';
-			if (fgets(q, sizeof q, stdin) == 0L)
+			if (fgets(q, sizeof q, stdin) == NULL)
 				continue;
 			switch (q[0]) {
 			case 'y':
@@ -743,6 +768,7 @@ static int replace_cmd(void) {
 	uid_t file_owner;
 	char **envp;
 	char *safename;
+	int envs = 0, entries = 0;
 
 
 	safename = host_specific_filename("#tmp", "XXXXXXXXXX");
@@ -792,15 +818,14 @@ static int replace_cmd(void) {
 		error = -2;
 		goto done;
 	}
-	fflush(tmp);
-	rewind(tmp);
-	if (ferror(tmp)) {
+	if (ferror(tmp) || fflush(tmp) || fsync(fileno(tmp))) {
 		fprintf(stderr, "%s: error while writing new crontab to %s\n",
 			ProgramName, TempFilename);
 		fclose(tmp);
 		error = -2;
 		goto done;
 	}
+	rewind(tmp);
 
 	/* check the syntax of the file being installed.
 	 */
@@ -822,6 +847,10 @@ static int replace_cmd(void) {
 	}
 
 	while (!CheckErrorCount && !eof) {
+		if (!skip_comments(tmp)) {
+			check_error("too many garbage characters");
+			break;
+		}
 		switch (load_env(envstr, tmp)) {
 		case ERR:
 			/* check for data before the EOF */
@@ -833,14 +862,30 @@ static int replace_cmd(void) {
 			break;
 		case FALSE:
 			e = load_entry(tmp, check_error, pw, envp);
-			if (e)
+			if (e) {
+				++entries;
 				free_entry(e);
+			}
 			break;
 		case TRUE:
+			++envs;
 			break;
 		}
 	}
 	env_free(envp);
+	if (envs > MAX_USER_ENVS) {
+		fprintf(stderr, "More than %d environment variables in crontab file, can't install.\n", MAX_USER_ENVS);
+		fclose(tmp);
+		error = -1;
+		goto done;
+	}
+
+	if (entries > MAX_USER_ENTRIES) {
+		fprintf(stderr, "More than %d entries in crontab file, can't install.\n", MAX_USER_ENTRIES);
+		fclose(tmp);
+		error = -1;
+		goto done;
+	}
 
 	if (CheckErrorCount != 0) {
 		fprintf(stderr, "errors in crontab file, can't install.\n");
@@ -852,14 +897,14 @@ static int replace_cmd(void) {
 	file_owner = (getgid() == geteuid() && getgid() == getegid()) ? ROOT_UID : pw->pw_uid;
 
 #ifdef HAVE_FCHOWN
-	if (fchown(fileno(tmp), file_owner, -1) < OK) {
+	if (fchown(fileno(tmp), file_owner, (gid_t)-1) < OK) {
 		perror("fchown");
 		fclose(tmp);
 		error = -2;
 		goto done;
 	}
 #else
-	if (chown(TempFilename, file_owner, -1) < OK) {
+	if (chown(TempFilename, file_owner, (gid_t)-1) < OK) {
 		perror("chown");
 		fclose(tmp);
 		error = -2;
@@ -913,7 +958,7 @@ static int hostset_cmd(void) {
 	
 	safename = host_specific_filename("#tmp", "XXXXXXXXXX");
 	if (!safename || !glue_strings(TempFilename, sizeof TempFilename, SPOOL_DIR,
-		safename, '/')) {
+			safename, '/')) {
 		TempFilename[0] = '\0';
 		fprintf(stderr, "path too long\n");
 		return (-2);
@@ -984,7 +1029,7 @@ static int hostget_cmd(void) {
 			fprintf(stderr, "File %s not found\n", n);
 		else
 			perror(n);
-			return (-2);
+		return (-2);
 	}
 
 	if (get_string(Host, sizeof Host, f, "\n") == EOF) {
@@ -1010,7 +1055,7 @@ static void poke_daemon(void) {
 	}
 }
 
-static void die(int x) {
+static void die(int x ATTRIBUTE_UNUSED) {
 	if (TempFilename[0])
 		(void) unlink(TempFilename);
 	_exit(ERROR_EXIT);
