@@ -57,21 +57,21 @@ cron_conv(int num_msg, const struct pam_message **msgm,
 	struct pam_response **response ATTRIBUTE_UNUSED,
 	void *appdata_ptr ATTRIBUTE_UNUSED)
 {
-        int i;
+	int i;
 
-        for (i = 0; i < num_msg; i++) {
-                switch (msgm[i]->msg_style) {
-                        case PAM_ERROR_MSG:
-                        case PAM_TEXT_INFO:
-                                if (msgm[i]->msg != NULL) {
-                                        log_it("CRON", getpid(), "pam_message", msgm[i]->msg, 0);
-                                }
-                        break;
-                        default:
-                        break;
-                }
-        }
-        return (0);
+	for (i = 0; i < num_msg; i++) {
+		switch (msgm[i]->msg_style) {
+		case PAM_ERROR_MSG:
+		case PAM_TEXT_INFO:
+			if (msgm[i]->msg != NULL) {
+				log_it("CRON", getpid(), "pam_message", msgm[i]->msg, 0);
+			}
+			break;
+		default:
+			break;
+		}
+	}
+	return (0);
 }
 
 static const struct pam_conv conv = {
@@ -101,7 +101,11 @@ static int cron_get_job_range(user * u, security_context_t * ucontextp,
 
 void cron_restore_default_security_context(void) {
 #ifdef WITH_SELINUX
-	setexeccon(NULL);
+	if (is_selinux_enabled() <= 0)
+		return;
+	if (setexeccon(NULL) < 0)
+		log_it("CRON", getpid(), "ERROR",
+			"failed to restore SELinux context", 0);
 #endif
 }
 
@@ -116,7 +120,7 @@ int cron_set_job_security_context(entry *e, user *u ATTRIBUTE_UNUSED,
 		/* "minute-ly" job: Every minute for given hour/dow/month/dom. 
 		 * Ensure that these jobs never run in the same minute:
 		 */
-		minutely_time = time(0);
+		minutely_time = time(NULL);
 		Debug(DSCH, ("Minute-ly job. Recording time %lu\n", minutely_time));
 	}
 
@@ -165,7 +169,7 @@ int cron_set_job_security_context(entry *e, user *u ATTRIBUTE_UNUSED,
 
 	*jobenv = build_env(e->envp);
 
-	time_t job_run_time = time(0L);
+	time_t job_run_time = time(NULL);
 
 	if ((minutely_time > 0) && ((job_run_time / 60) != (minutely_time / 60))) {
 		/* if a per-minute job is delayed into the next minute 
@@ -187,10 +191,10 @@ int cron_set_job_security_context(entry *e, user *u ATTRIBUTE_UNUSED,
 	return 0;
 }
 
+#if defined(WITH_PAM)
 int cron_start_pam(struct passwd *pw) {
 	int retcode = 0;
 
-#if defined(WITH_PAM)
 	retcode = pam_start("crond", pw->pw_name, &conv, &pamh);
 	PAM_FAIL_CHECK;
 	retcode = pam_set_item(pamh, PAM_TTY, "cron");
@@ -199,10 +203,10 @@ int cron_start_pam(struct passwd *pw) {
 	PAM_FAIL_CHECK;
 	retcode = pam_setcred(pamh, PAM_ESTABLISH_CRED | PAM_SILENT);
 	PAM_FAIL_CHECK;
-#endif
 
 	return retcode;
 }
+#endif
 
 #if defined(WITH_PAM)
 static int cron_open_pam_session(struct passwd *pw) {
@@ -346,12 +350,12 @@ cron_get_job_range(user * u, security_context_t * ucontextp, char **jobenv) {
 
 	if (is_selinux_enabled() <= 0)
 		return 0;
-	if (ucontextp == 0L)
+	if (ucontextp == NULL)
 		return -1;
 
-	*ucontextp = 0L;
+	*ucontextp = NULL;
 
-	if ((range = env_get("MLS_LEVEL", jobenv)) != 0L) {
+	if ((range = env_get("MLS_LEVEL", jobenv)) != NULL) {
 		context_t ccon;
 		if (!(ccon = context_new(u->scontext))) {
 			log_it(u->name, getpid(), "context_new FAILED for MLS_LEVEL",
@@ -400,7 +404,7 @@ static int cron_change_selinux_range(user * u, security_context_t ucontext) {
 	if (is_selinux_enabled() <= 0)
 		return 0;
 
-	if (u->scontext == 0L) {
+	if (u->scontext == NULL) {
 		if (security_getenforce() > 0) {
 			log_it(u->name, getpid(), "NULL security context for user", "", 0);
 			return -1;
@@ -413,7 +417,7 @@ static int cron_change_selinux_range(user * u, security_context_t ucontext) {
 		}
 	}
 
-	if (strcmp(u->scontext, ucontext)) {
+	if (!ucontext || strcmp(u->scontext, ucontext)) {
 		if (!cron_authorize_range(u->scontext, ucontext)) {
 			if (security_getenforce() > 0) {
 # ifdef WITH_AUDIT
@@ -501,6 +505,7 @@ get_security_context(const char *name, int crontab_fd,
 		retval = get_default_context_with_level(seuser, level, NULL, &scontext);
 	}
 	else {
+		const char *current_user, *current_role;
 		if (getcon(&current_context_str) < 0) {
 			log_it(name, getpid(), "getcon FAILED", "", 0);
 			return (security_getenforce() > 0);
@@ -513,8 +518,9 @@ get_security_context(const char *name, int crontab_fd,
 			return (security_getenforce() > 0);
 		}
 
-		const char *current_user = context_user_get(current_context);
-		retval = get_default_context_with_level(current_user, level, NULL, &scontext);
+		current_user = context_user_get(current_context);
+		current_role = context_role_get(current_context);
+		retval = get_default_context_with_rolelevel(current_user, current_role, level, NULL, &scontext);
 
 		freecon(current_context_str);
 		context_free(current_context);
@@ -586,7 +592,7 @@ get_security_context(const char *name, int crontab_fd,
 void free_security_context(security_context_t * scontext) {
 	if (*scontext != NULL) {
 		freecon(*scontext);
-		*scontext = 0L;
+		*scontext = NULL;
 	}
 }
 #endif
@@ -669,3 +675,74 @@ static char **build_env(char **cronenv) {
 	return jobenv;
 #endif
 }
+
+/* int in_file(const char *string, FILE *file, int error)
+ *	return TRUE if one of the lines in file matches string exactly,
+ *	FALSE if no lines match, and error on error.
+ */
+static int in_file(const char *string, FILE * file, int error) {
+	char line[MAX_TEMPSTR];
+	char *endp;
+
+	if (fseek(file, 0L, SEEK_SET))
+		return (error);
+	while (fgets(line, MAX_TEMPSTR, file)) {
+		if (line[0] != '\0') {
+			endp = &line[strlen(line) - 1];
+			if (*endp != '\n')
+				return (error);
+			*endp = '\0';
+			if (0 == strcmp(line, string))
+				return (TRUE);
+		}
+	}
+	if (ferror(file))
+		return (error);
+	return (FALSE);
+}
+
+/* int allowed(const char *username, const char *allow_file, const char *deny_file)
+ *	returns TRUE if (allow_file exists and user is listed)
+ *	or (deny_file exists and user is NOT listed).
+ *	root is always allowed.
+ */
+int allowed(const char *username, const char *allow_file,
+	const char *deny_file) {
+	FILE *fp;
+	int isallowed;
+	char buf[128];
+
+	if (getuid() == 0)
+		return TRUE;
+	isallowed = FALSE;
+	if ((fp = fopen(allow_file, "r")) != NULL) {
+		isallowed = in_file(username, fp, FALSE);
+		fclose(fp);
+		if ((getuid() == 0) && (!isallowed)) {
+			snprintf(buf, sizeof (buf),
+				"root used -u for user %s not in cron.allow", username);
+			log_it("crontab", getpid(), "warning", buf, 0);
+			isallowed = TRUE;
+		}
+	}
+	else if ((fp = fopen(deny_file, "r")) != NULL) {
+		isallowed = !in_file(username, fp, FALSE);
+		fclose(fp);
+		if ((getuid() == 0) && (!isallowed)) {
+			snprintf(buf, sizeof (buf),
+				"root used -u for user %s in cron.deny", username);
+			log_it("crontab", getpid(), "warning", buf, 0);
+			isallowed = TRUE;
+		}
+	}
+#ifdef WITH_AUDIT
+	if (isallowed == FALSE) {
+		int audit_fd = audit_open();
+		audit_log_user_message(audit_fd, AUDIT_USER_START, "cron deny",
+			NULL, NULL, NULL, 0);
+		close(audit_fd);
+	}
+#endif
+	return (isallowed);
+}
+
